module HttpThreshold
  class Throttle
    MANDATORY_OPTIONS = [:limit, :period]

    attr_reader :name, :limit, :period, :block, :type
    def initialize(name, options)
      @name = name
      MANDATORY_OPTIONS.each do |opt|
        raise ArgumentError.new("Must pass #{opt.inspect} option") unless options[opt]
      end
      @limit  = options[:limit]
      @period = options[:period].to_i
    end

    def cache
      HttpThreshold::Client.cache
    end

    def lock_manager
      HttpThreshold::Client.lock_manager
    end

    # similar as method #[]
    def incr_count
      begin
        result = lock_manager.lock!("lock:#{name}", period) do
          if reach_limit?
            false
          else
            cache.count(threshold_key, period)
            true
          end
        end
        result
      rescue Redlock::LockError
        sleep 0.25
        retry
      end
    end

    def reach_limit?
      count = cache.read_count(threshold_key, period)
      count >= limit
    end

    def status
      count = cache.read_count(threshold_key, period)
      "#{count}/#{limit}"
    end

    def threshold_key
      "threshold:#{name}"
    end
  end
end
